// Data containers
Table data;
int rows,cols;
int hiddenCol;
int[][] dataMatrix;
int[] dataMatrixMax;
int[] dataMatrixMin;

// Layout parameters
final int cellWidthMax = 100;
final int cellWidthMin = 10;
int cellHeight = 6;
int cellXDist = 1;
int cellYDist = 2;
int sideDist = 60;
int middleDist = 30;
final int fontYOffset = -35;
final float labelRelX = 1.7;
final float labelRelY = 2.0;

// Color parameters
final int bgColor = 255;
final int bgFontColor = 200;
final int labelFontColor = 180;
final float gradientAngle = 97.7;
final int gradientMin = 2;
final int gradientMax = 255;

// Misc.
final int absMax = 2147483647;
PFont labelFont,bgFont;
String bgValUpper = "";
String bgValLower = "";

void setup() {
  
  // Be smooth yo
  smooth();
  
  // Set font
  labelFont = createFont("Futura",10);
  bgFont = createFont("Futura",280);
  
  // Load data
  data = loadTable("assignment2_data.csv", "header");
  rows = data.getRowCount();
  cols = data.getColumnCount();
  
  // Hidden column is the last column
  hiddenCol = cols-1;
  
  // Resize arrays
  dataMatrix = new int[rows][cols];
  dataMatrixMax = new int[cols];
  dataMatrixMin = new int[cols];

  // Read data into array
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      dataMatrix[i][j] = data.getInt(i,j);
      print(dataMatrix[i][j] + " ");
    }
    println();
  }
  
  // Initialize arrays for minimum and maximum values
  for (int j=0; j<cols; j++) {
    dataMatrixMax[j] = 0;
    dataMatrixMin[j] = absMax;      
  }
  
  // Find minimum and maximum values for each column
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {
      if (dataMatrix[i][j] < dataMatrixMin[j]) dataMatrixMin[j] = dataMatrix[i][j];
      if (dataMatrix[i][j] > dataMatrixMax[j]) dataMatrixMax[j] = dataMatrix[i][j];
    }
  }
  
  // Cut the hidden column
  cols-=1;
  
  // Dynamic size, first two columns are always minimm width and followed by middleDist
  size(2*(cellWidthMin+cellXDist)+middleDist+(cols-2)*(cellWidthMax+cellXDist)+2*sideDist,rows*(cellHeight+cellYDist)+2*sideDist);
  
}


void draw() {
  
  background(255);
  
  // Draw the background values
  textFont(bgFont);
  textAlign(CENTER,CENTER);
  fill(bgFontColor);
  // Position includes hardcoded offset beause Processing SUCKS at text positioning
  text(bgValUpper,width/2,(height/3)*1+fontYOffset);
  text(bgValLower,width/2,(height/3)*2+fontYOffset);
  
  // Reset background values in each draw cycle
  bgValUpper = "";
  bgValLower = "";
  
  for (int i=0; i<rows; i++) {
    for (int j=0; j<cols; j++) {

        noStroke();
        
        int cellWidth,x;
        
        // If we're not in the first two columns, resize relative to global column width
        if (j!=0 && j!=1) {
          cellWidth = int(map(float(dataMatrix[i][hiddenCol]),float(dataMatrixMin[hiddenCol]),float(dataMatrixMax[hiddenCol]),cellWidthMin,cellWidthMax));
          x = 2*(cellWidthMin+cellXDist)+middleDist+(j-2)*(cellWidthMax+cellXDist);}
        // If not, use minimum width
        else {
          cellWidth = cellWidthMin;
          x = j*(cellWidthMin+cellXDist);}
        
        int y = i*(cellHeight+cellYDist);
        
        // Map cell value to column minimum and maximum
        float c = map(float(dataMatrix[i][j]),float(dataMatrixMin[j]),float(dataMatrixMax[j]),gradientMin,gradientMax);
                
        // "Dollar bill green" - see http://www.colorhexa.com/85bb65
        // Inline conversion from degrees to 8 bit integer
        colorMode(HSB);
        fill(map(gradientAngle,0,360,0,255),c,255);
        colorMode(RGB);
       
        // If the mouse is inside the rectangle, show the corresponding values in the background
        if (mouseX >= sideDist+x && mouseX <= sideDist+x+cellWidth && mouseY >= sideDist+y && mouseY <= sideDist+y+cellHeight) {
          bgValUpper = str(dataMatrix[i][j]);
          // Show the lower values only if we're not in the first two columns
          if (j!=0 && j!=1) {bgValLower = str(dataMatrix[i][hiddenCol]);}
        }

        rect(sideDist+x,sideDist+y,cellWidth,cellHeight);
        
      }
  }
  
  textFont(labelFont);
  textAlign(LEFT,CENTER);
  fill(labelFontColor);
  
  // Horizontal labels
  int firstTwo = sideDist+2*(cellWidthMin+cellXDist)+middleDist;
  text("MARX", firstTwo,sideDist/labelRelX);
  text("KEYNES", firstTwo+1*(cellWidthMax+cellXDist),sideDist/labelRelX);
  text("CAPITAL", firstTwo+2*(cellWidthMax+cellXDist),sideDist/labelRelX);
  text("CRISIS", firstTwo+3*(cellWidthMax+cellXDist),sideDist/labelRelX);
  
  // Vertical labels
  pushMatrix();
    translate(sideDist/labelRelY,height/2);
    rotate(radians(270));
    textAlign(CENTER,CENTER);
    text("FOUR RECESSION RELATED KEYWORDS IN TITLES 2006-2012 AND TOTAL TITLES FOR DEWEY DECIMALS 330-339: ECONOMICS",0,0);
  popMatrix();
  
}
